@@ -18,12 +18,10 @@ from oslo_log import log as logging
 
 from searchlight.elasticsearch.plugins import base
 from searchlight.elasticsearch.plugins.cinder import serialize_cinder_snapshot
-from searchlight import i18n
+from searchlight.i18n import _LE, _LW
 
 
 LOG = logging.getLogger(__name__)
-_LW = i18n._LW
-_LE = i18n._LE
 
 
 class SnapshotHandler(base.NotificationBase):
@@ -45,6 +43,12 @@ class SnapshotHandler(base.NotificationBase):
             'snapshot.delete.end': self.delete,
         }
 
+    def get_log_fields(self, event_type, payload):
+        return (
+            ('id', payload.get('snapshot_id')),
+            ('volume_id', payload.get('volume_id'))
+        )
+
     def create_or_update(self, payload, timestamp):
         snapshot_id = payload['snapshot_id']
         LOG.debug("Updating cinder snapshot information for %s", snapshot_id)
@@ -58,12 +62,14 @@ class SnapshotHandler(base.NotificationBase):
 
     def delete(self, payload, timestamp):
         snapshot_id = payload['snapshot_id']
+        volume_id = payload['volume_id']
         LOG.debug("Deleting cinder snapshot information for %s", snapshot_id)
         if not snapshot_id:
             return
 
         try:
-            self.index_helper.delete_document({'_id': snapshot_id})
+            self.index_helper.delete_document({'_id': snapshot_id,
+                                               '_parent': volume_id})
         except Exception as exc:
             LOG.error(_LE(
                 'Error deleting snapshot %(snapshot_id)s '
