@@ -22,7 +22,6 @@ and spinning down the servers.
 """
 
 import atexit
-import copy
 import datetime
 import elasticsearch
 import httplib2
@@ -44,11 +43,11 @@ import fixtures
 from oslo_serialization import jsonutils
 # NOTE(jokke): simplified transition to py3, behaves like py2 xrange
 from six.moves import range
+from six.moves import urllib
 import testtools
 
 from searchlight.common import utils
 from searchlight.elasticsearch.plugins import utils as es_utils
-from searchlight.elasticsearch import ROLE_USER_FIELD
 from searchlight.tests import utils as test_utils
 
 
@@ -69,7 +68,6 @@ class Server(object):
                          passed from the FunctionalTestCase.
         :param port: The port to start a server up on.
         """
-        self.verbose = True
         self.debug = True
         self.no_venv = False
         self.test_dir = test_dir
@@ -244,8 +242,10 @@ class SearchServer(Server):
         self.policy_default_rule = 'default'
         self.property_protection_rule_format = 'roles'
 
+        self.service_policy_files = ''
+        self.service_policy_path = ''
+
         self.conf_base = """[DEFAULT]
-verbose = %(verbose)s
 debug = %(debug)s
 log_file = %(log_file)s
 api_limit_max = 1000
@@ -261,6 +261,10 @@ flavor = %(deployment_flavor)s
 [elasticsearch]
 hosts = 127.0.0.1:%(elasticsearch_port)s
 
+[service_policies]
+service_policy_files = %(service_policy_files)s
+service_policy_path = %(service_policy_path)s
+
 [api]
 # Hardcoding a single worker; the cleanup code can't deal with more
 workers = 0
@@ -327,22 +331,24 @@ class FunctionalTest(test_utils.BaseTestCase):
 
         self.tracecmd = tracecmd_osmap.get(platform.system())
 
-        conf_dir = os.path.join(self.test_dir, 'etc')
-        utils.safe_mkdirs(conf_dir)
-        self.copy_data_file('policy.json', conf_dir)
-        self.copy_data_file('property-protections.conf', conf_dir)
-        self.copy_data_file('property-protections-policies.conf', conf_dir)
-        self.property_file_roles = os.path.join(conf_dir,
+        self.conf_dir = os.path.join(self.test_dir, 'etc')
+        utils.safe_mkdirs(self.conf_dir)
+        self.copy_data_file('policy.json', self.conf_dir)
+        self.copy_data_file('property-protections.conf', self.conf_dir)
+        self.copy_data_file('property-protections-policies.conf',
+                            self.conf_dir)
+        self.property_file_roles = os.path.join(self.conf_dir,
                                                 'property-protections.conf')
         property_policies = 'property-protections-policies.conf'
-        self.property_file_policies = os.path.join(conf_dir,
+        self.property_file_policies = os.path.join(self.conf_dir,
                                                    property_policies)
-        self.policy_file = os.path.join(conf_dir, 'policy.json')
+        self.policy_file = os.path.join(self.conf_dir, 'policy.json')
 
         self.api_server = SearchServer(self.test_dir,
                                        self.api_port,
                                        self.policy_file,
                                        sock=search_sock)
+        self._additional_server_config()
 
         self.pid_files = [self.api_server.pid_file]
         self.files_to_destroy = []
@@ -361,11 +367,16 @@ class FunctionalTest(test_utils.BaseTestCase):
                               "api_port",
                               max_retries=3,
                               **self.__dict__.copy())
+        self.addCleanup(self.cleanup)
 
         self.initialized_plugins = {}
         self.configurePlugins()
-        self.images_plugin = self.initialized_plugins['OS::Glance::Image']
-        self.metadefs_plugin = self.initialized_plugins['OS::Glance::Metadef']
+
+    def _additional_server_config(self):
+        """Any additional configuration for the API server. This is run prior
+        to writing the server config file and starting the server.
+        """
+        pass
 
     def configurePlugins(self, include_plugins=None, exclude_plugins=()):
         """Specify 'exclude_plugins' or 'include_plugins' as a list of
@@ -373,10 +384,15 @@ class FunctionalTest(test_utils.BaseTestCase):
         """
         plugin_classes = {
             'glance': {'images': 'ImageIndex', 'metadefs': 'MetadefIndex'},
-            'nova': {'servers': 'ServerIndex'},
+            'nova': {'servers': 'ServerIndex',
+                     'hypervisors': 'HypervisorIndex',
+                     'flavors': 'FlavorIndex',
+                     'servergroups': 'ServerGroupIndex'},
             'cinder': {'volumes': 'VolumeIndex', 'snapshots': 'SnapshotIndex'},
             'neutron': {'networks': 'NetworkIndex', 'ports': 'PortIndex',
-                        'subnets': 'SubnetIndex', 'routers': 'RouterIndex'},
+                        'subnets': 'SubnetIndex', 'routers': 'RouterIndex',
+                        'floatingips': 'FloatingIPIndex',
+                        'security_groups': 'SecurityGroupIndex'},
             'swift': {'accounts': 'AccountIndex',
                       'containers': 'ContainerIndex',
                       'objects': 'ObjectIndex'},
@@ -386,10 +402,12 @@ class FunctionalTest(test_utils.BaseTestCase):
 
         plugins = include_plugins or (
             ('glance', 'images'), ('glance', 'metadefs'),
-            ('nova', 'servers'),
+            ('nova', 'servers'), ('nova', 'hypervisors'),
+            ('nova', 'flavors'), ('nova', 'servergroups'),
             ('cinder', 'volumes'), ('cinder', 'snapshots'),
             ('neutron', 'networks'), ('neutron', 'ports'),
             ('neutron', 'subnets'), ('neutron', 'routers'),
+            ('neutron', 'floatingips'), ('neutron', 'security_groups'),
             ('cinder', 'volumes'), ('cinder', 'snapshots'),
             ('swift', 'accounts'), ('swift', 'containers'),
             ('swift', 'objects'),
@@ -433,82 +451,30 @@ class FunctionalTest(test_utils.BaseTestCase):
             instance.prepare_index(index_name=index_name)
 
         # Create the aliases
-        index = es_utils.setup_alias(index_name, 'searchlight-search',
-                                     'searchlight-listener')
-
-        # Finally, set up mappings etc in elasticsearch
-        for instance in six.itervalues(self.initialized_plugins):
-            instance.initial_indexing(index_name=index, setup_data=False)
+        es_utils.setup_alias(index_name, 'searchlight-search',
+                             'searchlight-listener')
 
     def tearDown(self):
-        if not self.disabled:
-            self.cleanup()
-
         super(FunctionalTest, self).tearDown()
 
         self.api_server.dump_log('api_server')
 
-        for plugin_instance in self.initialized_plugins.values():
-            self.elastic_connection.indices.delete(
-                index=plugin_instance.alias_name_search,
-                ignore=404)
-            self.elastic_connection.indices.delete(
-                index=plugin_instance.alias_name_listener,
-                ignore=404)
-
-    def _index(self, index_name, doc_type, docs, tenant,
-               parent_id_field=None,
-               role_separation=False, refresh_index=True):
-        # TODO(sjmc7) This whole function needs to use the IndexingHelper.
-        # It duplicates a lot of the behavior poorly
-
-        def apply_role_separation(apply_to_docs):
-            if role_separation:
-                for doc in apply_to_docs:
-                    parent_id = parent_id_field and doc.get(parent_id_field)
-
-                    user_doc = copy.deepcopy(doc)
-                    user_doc[ROLE_USER_FIELD] = 'user'
-                    action = {'_id': user_doc['id'] + '_USER',
-                              '_source': user_doc, '_op_type': 'index'}
-                    if parent_id:
-                        action['_parent'] = parent_id + '_USER'
-                    yield action
-
-                    admin_doc = copy.deepcopy(doc)
-                    admin_doc[ROLE_USER_FIELD] = 'admin'
-                    action = {'_id': admin_doc['id'] + '_ADMIN',
-                              '_source': admin_doc, '_op_type': 'index'}
-                    if parent_id:
-                        action['_parent'] = parent_id + '_ADMIN'
-                    yield action
-
-            else:
-                for doc in apply_to_docs:
-                    parent_id = parent_id_field and doc.get(parent_id_field)
-                    doc = copy.deepcopy(doc)
-                    if ROLE_USER_FIELD not in doc:
-                        doc[ROLE_USER_FIELD] = ['admin', 'user']
-                    action = {'_id': doc['id'], '_source': doc,
-                              '_op_type': 'index'}
-                    if parent_id:
-                        action['_parent'] = parent_id
-                    yield action
-
-        if not isinstance(docs, list):
-            docs = [docs]
-
-        result = elasticsearch.helpers.bulk(
-            client=self.elastic_connection,
-            index=index_name,
-            doc_type=doc_type,
-            actions=apply_role_separation(docs))
+    def _index(self, plugin, docs, refresh_index=True):
+        """Index data exactly as the plugin would under searchlight-manage.
+        docs must be an iterable of whatever the plugin's 'get_objects' call
+        would return.
+        """
+        with mock.patch.object(plugin, 'get_objects', return_value=docs):
+            with mock.patch.object(plugin, 'child_plugins', return_value=[]):
+                if hasattr(plugin, 'get_rbac_objects'):
+                    rbac_mock = mock.patch.object(plugin, 'get_rbac_objects',
+                                                  return_value={})
+                    rbac_mock.start()
+                plugin.index_initial_data()
 
         if refresh_index:
             # Force elasticsearch to update its search index
-            self._flush_elasticsearch(index_name)
-
-        return result
+            self._flush_elasticsearch(plugin.alias_name_listener)
 
     def _flush_elasticsearch(self, index_name=None):
         self.elastic_connection.indices.flush(index_name)
@@ -520,19 +486,22 @@ class FunctionalTest(test_utils.BaseTestCase):
             "X-User-Id": "f9a41d13-0c13-47e9-bee2-ce4e8bfe958e",
             "X-Tenant-Id": tenant_id,
             "X-Roles": "member",
-            "Content-Type": "application/json"
+            "Content-Type": "application/json",
+            "Accept": "application/json"
         }
         base_headers.update(custom_headers)
         return base_headers
 
     def _request(self, method, uri, tenant, body=None,
-                 role="member", decode_json=True):
+                 role="member", decode_json=True, extra_headers={}):
         custom_headers = {
             "X-Tenant-Id": tenant,
             "X-Roles": role,
         }
+        custom_headers.update(extra_headers)
         headers = self._headers(tenant, custom_headers)
-
+        if not tenant:
+            del headers['X-Tenant-Id']
         kwargs = {
             "headers": headers
         }
@@ -549,18 +518,30 @@ class FunctionalTest(test_utils.BaseTestCase):
             content = jsonutils.loads(content)
         return response, content
 
-    def _search_request(self, body, tenant, role="member", decode_json=True):
+    def _search_request(self, body, tenant, role="member", decode_json=True,
+                        is_admin_project=None):
         """Conduct a search against all elasticsearch indices unless specified
         in `body`. Returns the response and json-decoded content.
         """
+        extra_headers = {}
+        if is_admin_project is not None:
+            extra_headers['X-Is-Admin-Project'] = str(is_admin_project)
+
         return self._request("POST", "/search", tenant, body,
-                             role, decode_json)
+                             role, decode_json, extra_headers)
 
     def _facet_request(self, tenant, doc_type=None, role="member",
-                       decode_json=True):
+                       decode_json=True, include_fields=None,
+                       exclude_options=None):
         url = "/search/facets"
+        params = {}
         if doc_type:
-            url += '?type=%s' % doc_type
+            params['type'] = doc_type
+        if include_fields is not None:
+            params['include_fields'] = "true" if include_fields else "false"
+        if exclude_options is not None:
+            params['exclude_options'] = "true" if exclude_options else "false"
+        url += '?' + urllib.parse.urlencode(params)
         return self._request("GET", url, tenant, role=role,
                              decode_json=decode_json)
 
@@ -574,9 +555,14 @@ class FunctionalTest(test_utils.BaseTestCase):
             es_response = jsonutils.loads(es_response)
         return [h["_source"] for h in es_response["hits"]["hits"]]
 
-    def _get_all_elasticsearch_docs(self):
-        es_url = "http://localhost:%s/_search" % (
-            self.api_server.elasticsearch_port)
+    def _get_all_elasticsearch_docs(self, indices=[]):
+        """Query ES and return all documents. The caller can specify a list
+           of indices to query. If the list is empty, we will query all
+            indices.
+        """
+        index = ','.join(indices)
+        es_url = "http://localhost:%s/%s/_search" % (
+            self.api_server.elasticsearch_port, index)
         response, content = httplib2.Http().request(es_url)
         self.assertEqual(200, response.status)
         return jsonutils.loads(content)
@@ -589,6 +575,25 @@ class FunctionalTest(test_utils.BaseTestCase):
         json_content = jsonutils.loads(content)
         return json_content
 
+    def _delete_elasticsearch_doc(self, index_name, doc_type, doc_id):
+        es_url = "http://localhost:%s/%s/%s/%s" % (
+            self.api_server.elasticsearch_port, index_name, doc_type, doc_id)
+
+        response, content = httplib2.Http().request(es_url, "DELETE")
+        json_content = jsonutils.loads(content)
+        return json_content
+
+    def _get_elasticsearch_aliases(self, indices):
+        """Return all aliases associated with a specified index(es). The
+           caller can specify a list of indcies. If the list is empty, we
+           will query all indices.
+        """
+        index = ','.join(indices)
+        es_url = "http://localhost:%s/%s/_aliases" % (
+            self.api_server.elasticsearch_port, index)
+        response, content = httplib2.Http().request(es_url)
+        return jsonutils.loads(content)
+
     def _load_fixture_data(self, name):
         base_dir = "searchlight/tests/functional/data"
         with open(os.path.join(base_dir, name), 'r') as f:
@@ -621,6 +626,14 @@ class FunctionalTest(test_utils.BaseTestCase):
             if os.path.exists(f):
                 os.unlink(f)
 
+        for plugin_instance in self.initialized_plugins.values():
+            self.elastic_connection.indices.delete(
+                index=plugin_instance.alias_name_search,
+                ignore=404)
+            self.elastic_connection.indices.delete(
+                index=plugin_instance.alias_name_listener,
+                ignore=404)
+
     def start_server(self,
                      server,
                      expect_launch,
@@ -654,7 +667,7 @@ class FunctionalTest(test_utils.BaseTestCase):
         self.launched_servers.append(server)
 
         launch_msg = self.wait_for_servers([server], expect_launch)
-        self.assertTrue(launch_msg is None, launch_msg)
+        self.assertIsNone(launch_msg, launch_msg)
 
     def start_with_retry(self, server, port_name, max_retries,
                          expect_launch=True,
@@ -687,7 +700,7 @@ class FunctionalTest(test_utils.BaseTestCase):
             else:
                 self.launched_servers.append(server)
                 break
-        self.assertTrue(launch_msg is None, launch_msg)
+        self.assertIsNone(launch_msg, launch_msg)
 
     def ping_server(self, port):
         """
@@ -818,9 +831,11 @@ class ElasticsearchWrapper(object):
             'index.number_of_replicas': 0,
             'network.host': '127.0.0.1',
             'http.port': self.elasticsearch_port,
-            'discovery.zen.ping.multicast.disabled': 'false',
+            'discovery.zen.ping.multicast.enabled': 'false',
             'path.data': elasticsearch_root_dir,
-            'action.auto_create_index': False,
+            'path.conf': elasticsearch_root_dir,
+            'path.logs': elasticsearch_root_dir,
+            'action.auto_create_index': 'false',
             'script.engine.groovy.inline.update': 'on'
         }
         # Set JVM options
@@ -862,7 +877,7 @@ class ElasticsearchWrapper(object):
                 pass
             time.sleep(5)
         else:
-            raise Exception("Elasticsearch failed to start ")
+            raise Exception("Elasticsearch failed to start")
 
 
 elasticsearch_wrapper = ElasticsearchWrapper()
